use strict;
use warnings;
use Module::Build;

my $class = Module::Build->subclass(code => <<'EOF');
   use File::Spec;
   use File::Copy;

   sub process_db_files {
      my $self = shift;
      umask(0000); # create world writable db for tests
      my $qdb = File::Spec->catfile(qw(lib ACME QuoteDB DB), 'quotes.db');
      my $_t = File::Spec->catfile(qw(blib lib ACME QuoteDB DB), 'quotes.db');
      copy($qdb, $_t);
   }


   sub ACTION_install {
       my $self = shift;
       $self->SUPER::ACTION_install(@_);
       my $ddir =  $self->install_destination('lib');
       my $file = 
           File::Spec->catfile(("$ddir", 'ACME', 'QuoteDB', 'DB'), 'quotes.db');
       #chown('THIS_UNIX_USER' $file);
       # create quotes db as world writable, yup, change it if you don't like
       # default install is 0444, i.e. readonly
       chmod(0666, $file);
   }
   
EOF

my $builder = $class->new(
    module_name         => 'ACME::QuoteDB',
    license             => 'perl',
    dist_author         => 'David Wright <david_v_wright@yahoo.com>',
    dist_version_from   => 'lib/ACME/QuoteDB.pm',
    build_requires => {
        'Test::More'       => 0.8,
        'Readonly'         => 1.03,
        'Data::Dumper'     => 2.121,
        'Carp'             => 1.04,
        #'criticism'        => 1.02,
        'version'          => 0.70,
        'aliased'          => 0.22,
        'File::Basename'   => 2.74,
        'DBD::SQLite'      => 1.14,
        'Class::DBI'       => '3.0.17',
        'HTML::TokeParser' => 2.37,
        'Text::CSV'        => 1.06, #1.10, # 1.13
        'Cwd'              => 3.25, #3.29
        'File::Spec'       => 3.2501, #3.29
        'File::Copy'       => 2.11,
        #'Test::Pod'       => 1.22
        #'Test::Pod::Coverage'  => 1.08
    },
    add_to_cleanup      => [ 'ACME-QuoteDB-*' ],
    create_makefile_pl => 'traditional',
);

$builder->add_build_element('db');
$builder->create_build_script();
